// plot:

// Synopsis: RLaB / PlotMTV interface

// Syntax: plot ( A )
//         plot ( A, plotset_str, curve_str )

// Description:

//   The plot function runs the PlotMTV program with the data in
//   matrix A as input.  If A is a column vector, then the data is
//   plotted versus the row numbers.  If A is a MxN matrix, then the
//   2:N columns of A are plotted versus the first column.  If A is a
//   list, then all of the numerical elements (matrices) are plotted
//   in the new PlotMTV process.

//   Once the PlotMTV process is started, the user can interact with
//   PlotMTV via the its own graphical user interface; further control
//   from rlab is not possible.

//   The optional arguments, plotset_str, and curve_str, are lists
//   that contain PlotMTV plotset and curve attributes. Original
//   copies of these lists must be obtained via the get_plotset() and
//   get_curve()functions; then the user can modify the list-elements,
//   and use the modified lists as arguments to plot().

//   A synopsis of plot-related functions follows:

//   plot3 ( DATA , plotset_str , curve_str )

//   plotmult ( DATALIST , plotset_str , curve_str )

//   plmesh ( LIST , plotset_str , curve_str , dataset_str )
//      LIST must have elements x, y, and z.

//   plcont ( LIST , plotset_str , curve_str , dataset_str )
//      LIST must have elements x, y, and z.

//   pltitle ( TITLE )
//   plsubtitle ( TITLE )
//   plcomment ( TITLE )
//   xlabel ( XLABEL )
//   ylabel ( YLABEL )
//   zlabel ( ZLABEL )

//   plimits ( xmin , xmax , ymin , ymax )

//   plstyle ( line_style )
//      Arrays of strings: "line", "point", and "line-point" are valid.

//   plpoint ( POINT )
//      Argument is an integer array of markertype values (1-10).

//   plline ( LINE )
//      Argument is an integer array of linetype values (1-10).

//   plwid ( WIDTH )
//      Integers greater than 1 increase line thickness.

//   plegend ( LEGEND_STRING_MATRIX )

//   plgrid ( )
//--------------------------------------------------------------------

#
# RLaB interface to the PlotMTV program.
# This interface mimics the Plplot interface to the degree possible
# in order to miniminze the learning curve, and maximize the usability
# of existing programs that use plotting functions.
#
# --------------------------------------------------------------------
# Default plot structure. Completley defines the
# attributes of the "default" CURVE2D plot.
# --------------------------------------------------------------------

static (PSTR)
static (curve_resources)
static (dataset_resources)

static (inf_array)
inf_array = [ inf(), inf(), inf(), inf(), inf(), ...
              inf(), inf(), inf(), inf(), inf() ];
inf_array = [inf_array, inf_array, inf_array, inf_array, inf_array];

static (null_array)
null_array = ["", "", "", "", "", "", "", "", "", ""];
null_array = [null_array, null_array, null_array, null_array, null_array];

# --------------------------------------------------------------------
# Internal plot functions.
# --------------------------------------------------------------------

static (write_plotset_resources)
static (write_curve_resources)
static (write_dataset_resources)
static (write_contour_xgrid)
static (write_contour_ygrid)
static (write_contour_mesh)

# --------------------------------------------------------------------

PSTR = << 
  options = "-geom 500x500 -clobber -nodate -pscolor";
  xlabel = "";
  ylabel = "";
  zlabel = "";
  toplabel = "";
  subtitle = "";
  comment = "";
  sidelabel = "True";
  sidelabellength = 0;
  grid = "False";
  equalscale = "False";
  fitpage = "True";
  xyratio = 0.75;
  xautorange = "True";
  xticks = 4;
  yautorange = "True";
  yticks = 4;
  zautorange = "True";
  zticks = 4;
  autorange = "True";
  xlog = "False";
  ylog = "False";
  zlog = "False";
  xscale = 1.0;
  yscale = 1.0;
  zscale = 1.0;
  xticklabel = "";
  yticklabel = "";
  zticklabel = "";
  innerticks = "False";
  xmin = inf_array;
  xmax = inf_array;
  ymin = inf_array;
  ymax = inf_array;
  zmin = inf_array;
  zmax = inf_array;
  xflip = "False";
  yflip = "False";
  xabs = "False";
  yabs = "False";
  zabs = "False";
  overlay = "True";
>>;

curve_resources = <<
  boundary = "False";
  linelabel = null_array;
  linewidth = ones (1,50);
  linetype = ones (50,1);
  linecolor = [1:10, 1:10, 1:10, 1:10, 1:10];
  markertype = zeros (1,50);
  markercolor = 1:50;
  markersize = ones (1,50);
  filltype = zeros (1,50);
  fillcolor = [1:10, 1:10, 1:10, 1:10, 1:10];
>>;

dataset_resources = <<
  contstyle = 3;
  contfill = "on";
  contlabel = "off";
  bitmap = "False";
  nsteps = 10;
  meshplot = "off";
  interpolate = 2;
>>;

# --------------------------------------------------------------------
# Give the user the internal plot structure.
# --------------------------------------------------------------------

get_plotset = function ()
{
  return PSTR;
};

get_dataset = function ()
{
  return dataset_resources;
};

# --------------------------------------------------------------------
# Give the user the internal plot structure.
# --------------------------------------------------------------------

get_curve = function ()
{
  return curve_resources;
};

# --------------------------------------------------------------------
# Plot a matrix of values.
# --------------------------------------------------------------------

plot = function ( data , plotset_str, curve_str )
{
  if (!exist (data)) { rerror ("plot: must supply data"); }
  if (!exist (plotset_str)) { plotset_str = PSTR; }
  if (!exist (curve_str)) { curve_str = curve_resources; }

  #
  # Check if data is a matrix, or list...
  #

  if (class (data) == "num")
  {
    #
    # Data is a matrix.
    #

    # Tempory file name for plotting...
    pfn = tmpnam();

    pcmd = "plotmtv " + plotset_str.options + " " + pfn + ...
           " 1>/dev/null 2>&1 &";

    #
    # Write out the commands and properties...
    #
    
    fprintf (pfn, "$ DATA=CURVE2D\n");
    
    #
    # If its a single column, we need to fix it...
    #
    
    if (data.nc == 1)
    {
      data = [ (1:data.nr)', data ];
    }

    #
    # Write out the data...
    #
    
    for (i in 2:data.nc)
    {
      fprintf (pfn, "\n");
      write_plotset_resources (pfn, i-1, plotset_str);
      write_curve_resources (pfn, i-1, curve_str);

      writem (pfn, data[;1,i]);

      # fprintf (pfn, "\n");
      #for (j in 1:data.nr)
      #{
      #fprintf (pfn, "%-20.10g %20.10g\n", data[j;1], data[j;i]);
      #}
    }
  
    fprintf (pfn, "$ END\n");
    close (pfn);

  else if (class (data) == "list") {
    #
    # Plot a LIST
    #

    pcmd = "plotmtv " + " -plotall " + plotset_str.options;

    j = 1;
    for (i in members (data))
    {
      if (class (data.[i]) == "num") 
      {
	pfn[j] = tmpnam ();
      
	# PlotMTV command
	pcmd = pcmd + " " + pfn[j];
	j++;
      else
	continue
      }
    }
    pcmd = pcmd  + " 1>/dev/null 2>&1 &";

    #
    # Write out the commands and properties to each file...
    #
    
    j = 1;
    for (i in members (data))
    {
      if (class (data.[i]) == "num")
      {
	fprintf (pfn[j], "$ DATA=CURVE2D\n");
	
	# Get the data...
	dtmp = data.[i];
	
	#
	# If its a single column, we need to fix it...
	#
	
	if (dtmp.nc == 1)
	{
	  dtmp = [ (1:dtmp.nr)', dtmp ];
	}
	
	#
	# Write out the data...
	#
	
	for (k in 2:dtmp.nc)
	{
	  fprintf (pfn[j], "\n");
	  write_plotset_resources (pfn[j], j, plotset_str);
	  write_curve_resources (pfn[j], j, curve_str);

	  writem (pfn[j], dtmp[;1,k]);

	  # fprintf (pfn[j], "\n");
	  # for (ii in 1:dtmp.nr)
	  # {
	  #  fprintf (pfn[j], "%-20.10g %20.10g\n", dtmp[ii;1], dtmp[ii;k]);
	  #}
	}
	
	fprintf (pfn[j], "$ END\n");
	close (pfn[j]);
	j++;
      }
    }

  } }

  #
  # Fire up plotmtv...
  #

  system (pcmd);
};


# --------------------------------------------------------------------
# Plot a matrix of values.
# --------------------------------------------------------------------

plot3 = function ( data , plotset_str, curve_str )
{
  if (!exist (data)) { rerror ("plot3: must supply data"); }
  if (!exist (plotset_str)) { plotset_str = PSTR; }
  if (!exist (curve_str)) { curve_str = curve_resources; }

  #
  # Check if data is a matrix, or list...
  #

  if (class (data) == "num")
  {
    #
    # Data is a matrix.
    #

    # Tempory file name for plotting...
    pfn = tmpnam();

    pcmd = "plotmtv " + plotset_str.options + " " + pfn + ...
           " 1>/dev/null 2>&1 &";

    #
    # Write out the commands and properties...
    #
    
    fprintf (pfn, "$ DATA=CURVE3D\n");
    
    #
    # Error check
    #
    
    if (data.nc != 3)
    {
      error ("plot3: input matrix must have 3 columns");
    }

    #
    # Write out the data...
    #
    
    write_plotset_resources (pfn, 1, plotset_str);
    write_curve_resources (pfn, 1, curve_str);
    
    writem (pfn, data);

    fprintf (pfn, "$ END\n");
    close (pfn);

  else if (class (data) == "list") {
    #
    # Plot a LIST
    #

    pcmd = "plotmtv " + " -plotall " + plotset_str.options;

    j = 1;
    for (i in members (data))
    {
      if (class (data.[i]) == "num") 
      {
	pfn[j] = tmpnam ();
      
	# PlotMTV command
	pcmd = pcmd + " " + pfn[j];
	j++;
      else
	continue
      }
    }
    pcmd = pcmd  + " 1>/dev/null 2>&1 &";

    #
    # Write out the commands and properties to each file...
    #
    
    j = 1;
    for (i in members (data))
    {
      if (class (data.[i]) == "num")
      {
	fprintf (pfn[j], "$ DATA=CURVE3D\n");
	
	# Get the data...
	dtmp = data.[i];
	
	#
	# If its a single column, we need to fix it...
	#
	
	if (dtmp.nc != 3)
	{
	  error ("plot3: input matrix must have 3 columns");
	}
	
	#
	# Write out the data...
	#
	
	fprintf (pfn[j], "\n");
	write_plotset_resources (pfn[j], j, plotset_str);
	write_curve_resources (pfn[j], j, curve_str);
	
	writem (pfn[j], dtmp);

	fprintf (pfn[j], "$ END\n");
	close (pfn[j]);
	j++;
      }
    }

  } }

  #
  # Fire up plotmtv...
  #

  system (pcmd);
};

# --------------------------------------------------------------------
# Plot multiple plots in the same window
# --------------------------------------------------------------------

plotmult = function ( LIST , plot_str, curve_str )
{
  if (!exist (LIST)) { 
    error ("plotmult: must supply a list argument"); 
  }
  if (class (LIST) != "list") {
    error ("plotmult: 1st argument must be a list"); 
  }
  if (!exist (plot_str)) { 
    plot_str = PSTR; 
  }
  if (!exist (curve_str)) { 
    curve_str = curve_resources; 
  }
  
  # Construct the plot-command...
  pcmd = "plotmtv " + " -mult " + plot_str.options;
  j = 1;
  for (i in members (LIST))
  {
    if (class (LIST.[i]) == "num")
    {
      pfn[j] = tmpnam();
      
      # PlotMTV command
      pcmd = pcmd + " " + pfn[j];
      j++;
      else
      continue
    }
  }
  pcmd = pcmd  + " 1>/dev/null 2>&1 &";
  
  #
  # Write out the commands and properties to each file...
  #
  
  j = 1;
  for (i in members (LIST))
  {
    if (class (LIST.[i]) == "num")
    {
      fprintf (pfn[j], "$ DATA=CURVE2D\n");
      
      # Get the data...
      data = LIST.[i];
      
      #
      # If its a single column, we need to fix it...
      #
      
      if (data.nc == 1)
      {
	data = [ (1:data.nr)', data ];
      }
      
      #
      # Write out the data...
      #
      
      for (k in 2:data.nc)
      {
	fprintf (pfn[j], "\n");
	write_plotset_resources (pfn[j], k-1, plot_str);
	write_curve_resources (pfn[j], k-1, curve_str);

	writem (pfn[j], data[;1,k]);

	# fprintf (pfn[j], "\n");
	#for (ii in 1:data.nr)
	#{
	#  fprintf (pfn[j], "%-20.10g %20.10g\n", data[ii;1], data[ii;k]);
	#}
      }
      
      fprintf (pfn[j], "$ END\n");
      close (pfn[j]);
      j++;
    }
  }

  #
  # Fire up plotmtv...
  #
  
  system (pcmd);
};

# --------------------------------------------------------------------
# Plot a 3D Mesh
# --------------------------------------------------------------------

plmesh = function ( LIST , plotset_str, curve_str, dataset_str )
{
  if (!exist (plotset_str)) { plotset_str = PSTR; }
  if (!exist (curve_str)) { curve_str = curve_resources; }
  if (!exist (dataset_str)) { dataset_str = dataset_resources; }

  # Tempory file name for plotting...
  pfn = tmpnam();

  # PlotMTV command
  pcmd = "plotmtv " + "-3D" + plotset_str.options + " " + pfn + ...
         " 1>/dev/null 2>&1 &";

  #
  # Write out the commands and properties...
  #

  fprintf (pfn, "$ DATA=CONTOUR\n");

  #
  # Write out the data...
  #

  fprintf (pfn, "\n");
  write_plotset_resources (pfn, 1, plotset_str);
  fprintf (pfn, "%s contstyle = 3\n", "%");  

  write_contour_xgrid (pfn, LIST.x);
  write_contour_ygrid (pfn, LIST.y);
  write_contour_mesh (pfn, LIST.z);

  fprintf (pfn, "$ END\n");
  close (pfn);

  #
  # Fire up plotmtv...
  #

  system (pcmd);
};

# --------------------------------------------------------------------
# Plot a contour
# --------------------------------------------------------------------

plcont = function ( LIST , plotset_str, curve_str, dataset_str )
{
  if (!exist (plotset_str)) { plotset_str = PSTR; }
  if (!exist (curve_str)) { curve_str = curve_resources; }
  if (!exist (dataset_str)) { dataset_str = dataset_resources; }

  # Tempory file name for plotting...
  pfn = tmpnam();

  # PlotMTV command
  pcmd = "plotmtv " + plotset_str.options + " " + pfn + ...
         " 1>/dev/null 2>&1 &";

  #
  # Write out the commands and properties...
  #

  fprintf (pfn, "$ DATA=CONTOUR\n");

  #
  # Write out the data...
  #

  fprintf (pfn, "\n");
  write_plotset_resources (pfn, 1, plotset_str);
  write_dataset_resources (pfn, 1, dataset_str);

  write_contour_xgrid (pfn, LIST.x);
  write_contour_ygrid (pfn, LIST.y);
  write_contour_mesh (pfn, LIST.z);

  fprintf (pfn, "$ END\n");
  close (pfn);

  #
  # Fire up plotmtv...
  #

  system (pcmd);
};

# --------------------------------------------------------------------
# Write the PLOTSET Resources
# See the PlotMTV manual.
# --------------------------------------------------------------------

write_plotset_resources = function ( pfn, i, plot_str )
{
  fprintf (pfn, "%s xlabel = \"%s\"\n", "%", plot_str.xlabel);
  fprintf (pfn, "%s ylabel = \"%s\"\n", "%", plot_str.ylabel);
  fprintf (pfn, "%s zlabel = \"%s\"\n", "%", plot_str.zlabel);
  fprintf (pfn, "%s toplabel = \"%s\"\n", "%", plot_str.toplabel);
  fprintf (pfn, "%s subtitle = \"%s\"\n", "%", plot_str.subtitle);
  fprintf (pfn, "%s comment = \"%s\"\n", "%", plot_str.comment);
  fprintf (pfn, "%s sidelabel = \"%s\"\n", "%", plot_str.sidelabel);
  fprintf (pfn, "%s sidelabellength = \%i\n", "%", plot_str.sidelabellength);

  fprintf (pfn, "%s grid = \"%s\"\n", "%", plot_str.grid);
  fprintf (pfn, "%s equalscale = \"%s\"\n", "%", plot_str.equalscale);
  fprintf (pfn, "%s fitpage = \"%s\"\n", "%", plot_str.fitpage);
  fprintf (pfn, "%s xyratio = \"%f\"\n", "%", plot_str.xyratio);

  fprintf (pfn, "%s xautorange = \"%s\"\n", "%", plot_str.xautorange);
  fprintf (pfn, "%s xticks = \"%i\"\n", "%", plot_str.xticks);
  fprintf (pfn, "%s yautorange = \"%s\"\n", "%", plot_str.yautorange);
  fprintf (pfn, "%s yticks = \"%i\"\n", "%", plot_str.yticks);
  fprintf (pfn, "%s zautorange = \"%s\"\n", "%", plot_str.zautorange);
  fprintf (pfn, "%s zticks = \"%i\"\n", "%", plot_str.zticks);
  fprintf (pfn, "%s autorange = \"%s\"\n", "%", plot_str.autorange);
  fprintf (pfn, "%s xlog = \"%s\"\n", "%", plot_str.xlog);
  fprintf (pfn, "%s ylog = \"%s\"\n", "%", plot_str.ylog);
  fprintf (pfn, "%s zlog = \"%s\"\n", "%", plot_str.zlog);
  fprintf (pfn, "%s xscale = \"%f\"\n", "%", plot_str.xscale);
  fprintf (pfn, "%s yscale = \"%f\"\n", "%", plot_str.yscale);
  fprintf (pfn, "%s zscale = \"%f\"\n", "%", plot_str.zscale);
  fprintf (pfn, "%s xticklabel = \"%s\"\n", "%", plot_str.xticklabel);
  fprintf (pfn, "%s yticklabel = \"%s\"\n", "%", plot_str.yticklabel);
  fprintf (pfn, "%s zticklabel = \"%s\"\n", "%", plot_str.zticklabel);
  fprintf (pfn, "%s innerticks = \"%s\"\n", "%", plot_str.innerticks);

  if (!isinf (plot_str.xmin[i])) 
  {
    fprintf (pfn, "%s xmin = %f\n", "%", plot_str.xmin[i]);
  }

  if (!isinf (plot_str.xmax[i])) 
  {
    fprintf (pfn, "%s xmax = %f\n", "%", plot_str.xmax[i]);
  }

  if (!isinf (plot_str.ymin[i])) 
  {
    fprintf (pfn, "%s ymin = %f\n", "%", plot_str.ymin[i]);
  }

  if (!isinf (plot_str.ymax[i])) 
  {
    fprintf (pfn, "%s ymax = %f\n", "%", plot_str.ymax[i]);
  }

  if (!isinf (plot_str.zmin[i])) 
  {
    fprintf (pfn, "%s zmin = %f\n", "%", plot_str.zmin[i]);
  }

  if (!isinf (plot_str.zmax[i])) 
  {
    fprintf (pfn, "%s zmax = %f\n", "%", plot_str.zmax[i]);
  }

  fprintf (pfn, "%s xflip = \"%s\"\n", "%", plot_str.xflip);
  fprintf (pfn, "%s yflip = \"%s\"\n", "%", plot_str.yflip);
  fprintf (pfn, "%s xabs = \"%s\"\n", "%", plot_str.xabs);
  fprintf (pfn, "%s yabs = \"%s\"\n", "%", plot_str.yabs);
  fprintf (pfn, "%s zabs = \"%s\"\n", "%", plot_str.zabs);
  fprintf (pfn, "%s overlay = \"%s\"\n", "%", plot_str.overlay);
};

# --------------------------------------------------------------------
# Write local (per line) plot instructions.
# --------------------------------------------------------------------

write_curve_resources = function ( pfn, i, plot_str)
{
  fprintf (pfn, "%s boundary = \"%s\"\n", "%", plot_str.boundary);  
  fprintf (pfn, "%s linelabel = \"%s\"\n", "%", plot_str.linelabel[i]);  
  fprintf (pfn, "%s linewidth = %i\n","%",  plot_str.linewidth[i]);  
  fprintf (pfn, "%s linetype = %i\n", "%", plot_str.linetype[i]);  
  fprintf (pfn, "%s linecolor = %i\n", "%", plot_str.linecolor[i]);  

  fprintf (pfn, "%s markertype = %i\n", "%", plot_str.markertype[i]);  
  fprintf (pfn, "%s markercolor = %i\n", "%", plot_str.markercolor[i]);  
  fprintf (pfn, "%s markersize = %i\n", "%", plot_str.markersize[i]);  

  fprintf (pfn, "%s filltype = \"%i\"\n", "%", plot_str.filltype[i]);  
  fprintf (pfn, "%s fillcolor = \"%i\"\n", "%", plot_str.fillcolor[i]);  
};

# --------------------------------------------------------------------
# Write dataset resources
# --------------------------------------------------------------------

write_dataset_resources = function ( pfn, i, plot_str)
{
#  fprintf (pfn, "%s contstyle = %i\n", "%", plot_str.contstyle);  
  fprintf (pfn, "%s contfill = \"%s\"\n","%",  plot_str.contfill);  
  fprintf (pfn, "%s contlabel = \"%s\"\n","%",  plot_str.contlabel);  
  fprintf (pfn, "%s nsteps = %i\n", "%", plot_str.nsteps);  
  fprintf (pfn, "%s meshplot = \"%s\"\n","%",  plot_str.meshplot);  
  fprintf (pfn, "%s interpolate = %i\n", "%", plot_str.interpolate);  
};

# --------------------------------------------------------------------
# Write out the contour grid (X or Y)
# -------------------------------------------------------------------

write_contour_xgrid  = function (pfn, grid)
{
  fprintf (pfn, "%s nx = %i\n", "%", grid.n);
  fprintf (pfn, "%s xgrid = True\n", "%");
  for (i in 1:grid.n)
  {
    fprintf (pfn, "%f\n", grid[i]);
  }
};

write_contour_ygrid  = function (pfn, grid)
{
  fprintf (pfn, "%s ny = %i\n", "%", grid.n);
  fprintf (pfn, "%s ygrid = True\n", "%");
  for (i in 1:grid.n)
  {
    fprintf (pfn, "%f\n", grid[i]);
  }
};

write_contour_mesh  = function (pfn, mesh)
{
  fprintf (pfn, "\n");
  for (i in 1:mesh.nr)
  {
    for (j in 1:mesh.nc)
    {
      fprintf (pfn, "%f\n", mesh[i;j]);
    }
  }
};

# --------------------------------------------------------------------
# Set the Plot Title
# --------------------------------------------------------------------

pltitle = function ( TITLE )
{
  if (!exist (TITLE)) 
  { 
    TITLE = ""; 
  }
  PSTR.toplabel = TITLE;
};

plsubtitle = function ( TITLE )
{
  if (!exist (TITLE)) 
  { 
    TITLE = ""; 
  }
  PSTR.subtitle = TITLE;
};

plcomment = function ( TITLE )
{
  if (!exist (TITLE)) 
  { 
    TITLE = ""; 
  }
  PSTR.comment = TITLE;
};

# --------------------------------------------------------------------
# Set the X-Axis Label
# --------------------------------------------------------------------

xlabel = function ( XLABEL )
{
  if (!exist (XLABEL)) 
  { 
    XLABEL = ""; 
  }
  PSTR.xlabel = XLABEL;
};

# --------------------------------------------------------------------
# Set the Y-Axis Label
# --------------------------------------------------------------------

ylabel = function ( YLABEL )
{
  if (!exist (YLABEL)) 
  { 
    YLABEL = ""; 
  }
  PSTR.ylabel = YLABEL;
};

# --------------------------------------------------------------------
# Set the Z-Axis Label
# --------------------------------------------------------------------

zlabel = function ( ZLABEL )
{
  if (!exist (ZLABEL)) 
  { 
    ZLABEL = ""; 
  }
  PSTR.zlabel = ZLABEL;
};

# --------------------------------------------------------------------
# Set the plot scale limits.
# --------------------------------------------------------------------

plimits = function ( xmin, xmax, ymin, ymax )
{
  if (!exist (xmin))
  {
    PSTR.xmin = inf_array;
  else
    for (i in 1:xmin.n)
    {
      PSTR.xmin[i] = xmin[i];
    }
  }
    
  if (!exist (xmax))
  {
    PSTR.xmax = inf_array;
  else
    for (i in 1:xmax.n)
    {
      PSTR.xmax[i] = xmax[i];
    }
  }
    
  if (!exist (ymin))
  {
    PSTR.ymin = inf_array;
  else
    for (i in 1:ymin.n)
    {
      PSTR.ymin[i] = ymin[i];
    }
  }
    
  if (!exist (ymax))
  {
    PSTR.ymax = inf_array;
  else
    for (i in 1:ymax.n)
    {
      PSTR.ymax[i] = ymax[i];
    }
  }
};

# --------------------------------------------------------------------
# Set line style
# Acceptable inputs are: "line" 
#                        "point"
#                        "line-point"
# --------------------------------------------------------------------

plstyle = function ( line_style )
{
  if (!exist (line_style)) 
  {
    line_style = "line";
  }

  if (line_style.n == 1)
  {
    if (line_style == "line")
    {
      curve_resources.linetype =  [1:10, 1:10, 1:10, 1:10, 1:10];
      curve_resources.markertype = zeros (1,50);
    }
    
    if (line_style == "point")
    {
      curve_resources.linetype = zeros (1,50);
      curve_resources.markertype = [2:11, 2:11, 2:11, 2:11, 2:11];
    }    
    
    if (line_style == "line-point")
    {
      curve_resources.linetype =  [1:10, 1:10, 1:10, 1:10, 1:10];
      curve_resources.markertype =  [2:11, 2:11, 2:11, 2:11, 2:11];
    }    
  else

    line_mask = ( "line" ==  line_style );
    point_mask = ( "point" == line_style );
    linepoint_mask = ( "line-point" == line_style );
    
    curve_resources.linetype = (line_mask || linepoint_mask);
    curve_resources.markertype = (point_mask || linepoint_mask);
    curve_resources.markertype.n;
    curve_resources.markertype = curve_resources.markertype .* ...
                                 (2:curve_resources.markertype.n+1);

  }
};

# --------------------------------------------------------------------
# Set the point types (markertype)
# ARGS:
#    POINT:  A 10 element array containing integers that denote marker-style. 
#            See the PlotMTV Reference Manual
# --------------------------------------------------------------------

plpoint = function ( POINT )
{
  if (!exist (POINT)) 
  {
    POINT = zeros (1,50);
  }

  for (i in 1:POINT.n)
  {
    curve_resources.markertype[i] = POINT[i];
  }
};

# --------------------------------------------------------------------
# Set the point types (markertype)
# ARGS:
#    POINT:  A 10 element array containing integers that denote marker-style. 
#            See the PlotMTV Reference Manual
# --------------------------------------------------------------------

plline = function ( LINE )
{
  if (!exist (LINE)) 
  {
    LINE = zeros (1,10);
  }

  for (i in 1:LINE.n)
  {
    curve_resources.linetype[i] = LINE[i];
  }
};

# --------------------------------------------------------------------
# Set the line width
# ARGS:
#    WIDTH: 1 is the default, > 1 is thicker.
# --------------------------------------------------------------------

plwid  = function ( WIDTH )
{
  if (!exist (WIDTH))
  {
    WIDTH = ones (1,10);
  }

  for (i in 1:WIDTH.n)
  {
    curve_resources.linewidth[i] = WIDTH[i];
  }
};

plmsize  = function ( WIDTH )
{
  if (!exist (WIDTH))
  {
    WIDTH = ones (1,10);
  }

  for (i in 1:WIDTH.n)
  {
    curve_resources.markersize[i] = WIDTH[i];
  }
};

# --------------------------------------------------------------------
# Set the LineLabels (legend)
# --------------------------------------------------------------------

plegend = function ( legend )
{
  if (!exist (legend))
  {
    curve_resources.linelabel = ["", "", "", "", "", "", "", "", "", ""];
    return 1;
  }

  for (i in 1:legend.n)
  {
    curve_resources.linelabel[i] = legend[i];
  }
};

# --------------------------------------------------------------------
# Set the grid attributes.
# --------------------------------------------------------------------

plgrid = function ( TF )
{
  if (!exist (TF))
  {
    TF = "False";
  }

  PSTR.grid = TF;
};

# --------------------------------------------------------------------
# Set the boundary attribute.
# --------------------------------------------------------------------

plboundary = function ( TF )
{
  if (!exist (TF))
  {
    TF = "False";
  }

  PSTR.boundary = TF;
};

# --------------------------------------------------------------------
# Start a plot window.
# This is a DUMMY function.
# --------------------------------------------------------------------

plstart = function ( dummy1, dummy2 )
{
  a = 1;
};

# --------------------------------------------------------------------
# Set the plot font
# This is a DUMMY function.
# --------------------------------------------------------------------

plfont = function ( dummy1 )
{
  a = 1;
};
